/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "quaqut.h"
#include "protocol.h"

void quaqut_display_instructions( void )
{
  printf( "\nUSAGE:   quaqut [-param [argument]] host\n"
	  "\nParameters are listed below:\n\n"
	  "\t-a\t(default) display complete information\n"
	  "\t-s\tdisplay only server information\n"
	  "\t-g\tdisplay only game information\n"
	  "\t-y\tdisplay only players information\n"
	  "\t-p port\tspecify game port for server\n"
	  "\t-q port\tspecify query port for server\n"
	  "\t-t nnn\tset a custom timeout to wait server reply, nnn\n"
	    "\t\tis in tenths of second and can vary from 1 to 999\n"
	  "\t-d nnnn\tquaqut daemon mode (keeps querying), nnnn is the\n"
	    "\t\tinterval in seconds between queries (from 10 to 9999)\n"
	  "\t-f fln\twrite information to file fln instead of printing\n"
	    "\t\tthem to standard output (example -f /home/bar/foo.log)\n"
	  "\t-r\tdisplay output in raw format\n"
	  "\t-v\tdisplay quaqut version\n\n"
	  );
}

void quaqut_display_error( int error_code , char* message , char* extra )
{
  if ( error_code == 0 ) /* it is a custom error message */
  {
    printf( message , extra );
  }
  else /* we have a preset message to display */
  {
    switch (error_code)
    {
      case QUAQUT_ERROR_INVALIDQUERYTIMEOUT:
        printf( "Error: invalid timeout value after parameter -t\n" );
	break;
      case QUAQUT_ERROR_INVALIDDAEMONINTERVAL:
        printf( "Error: invalid interval value after parameter -d\n" );
	break;
      case QUAQUT_ERROR_MISSINGFILE:
        printf( "Error: missing log file after parameter -f\n" );
	break;
      case QUAQUT_ERROR_MISSINGQUERYTIMEOUT:
        printf( "Error: missing timeout value after parameter -t\n" );
	break;
      case QUAQUT_ERROR_MISSINGDAEMONINTERVAL:
        printf( "Error: missing interval value after parameter -d\n" );
	break;
      case QUAQUT_ERROR_MISSINGHOSTNAME:
        printf( "Error: missing hostname\n" );
	break;
      case QUAQUT_ERROR_MISSINGPORT:
        printf( "Error: missing port number\n" );
	break;
    }
  }
}

void quaqut_display_version( void )
{
  printf( "quaqut %s , written by Giacomo Lozito\nquaqut is released under GNU GPL (version 2)\n" , VERSION );
}

int quaqut_display_process( int protocol , char command )
{
  /* Determine where we are going to write ouput (stdout or file) */
  FILE *destination;
  if ( quaqut.options & QUAQUT_OPTION_LOGFILE )
  {
    destination = fopen( quaqut.output_filename , "w" );
    if ( !destination )
    {
      quaqut_display_error( QUAQUT_ERROR_FILEPROBLEM , NULL , NULL );
      return 0;
    }
  }
  else destination = stdout;

  switch ( command )
  {
    case 'a':
      quaqut_display_information( destination , QUAQUT_PROTOCOL_UT2004 , QUAQUT_SERVERINFO );
      quaqut_display_information( destination , QUAQUT_PROTOCOL_UT2004 , QUAQUT_GAMEINFO );
      if ( quaqut.buffer_reply_size[QUAQUT_PLAYERINFO] > 0 )
        quaqut_display_information( destination , QUAQUT_PROTOCOL_UT2004 , QUAQUT_PLAYERINFO );
      break;
    
    case 's':
      quaqut_display_information( destination , QUAQUT_PROTOCOL_UT2004 , QUAQUT_SERVERINFO );
      break;
    
    case 'g':
      quaqut_display_information( destination , QUAQUT_PROTOCOL_UT2004 , QUAQUT_GAMEINFO );
      break;
    
    case 'y':
      quaqut_display_information( destination , QUAQUT_PROTOCOL_UT2004 , QUAQUT_PLAYERINFO );
      break;
  }
  
  if ( quaqut.options & QUAQUT_OPTION_LOGFILE )
    fclose(destination);
  
  return 1;
}

void quaqut_display_information( FILE* destination , int protocol , int request )
{
  if ( quaqut.options & QUAQUT_OPTION_RAWINFO )
  {
    /* Raw format can be useful if users want to
       organize ouput within their frontend or whatever */
    fprintf( destination , "%s\n%s" , quaqut_raw_label[protocol][request] , quaqut.buffer_reply[request] );
  }
  else
  {
    /* Nicely formatted output :)
       (which depends on game protocol) */
    
    switch ( protocol )
    {
      case QUAQUT_PROTOCOL_UT2004:
        quaqut_display_information_ut2004( destination , request );
        break;    
    }
  }
}

void quaqut_display_information_ut2004( FILE* destination , int request )
{
  switch ( request )
  {
    case QUAQUT_SERVERINFO:
    {
      if ( quaqut.buffer_reply_entriesnum[request] > 0 )
      {
        char* token[quaqut.buffer_reply_entriesnum[request]];
        int i;
	
	token[0] = (char*)quaqut_utils_strsep( quaqut.buffer_reply[request] , '\n' );
        for ( i = 1 ; i < quaqut.buffer_reply_entriesnum[request] ; i++ )
          token[i] = (char*)quaqut_utils_strsep( NULL , '\n' );
        
	fprintf( destination , "\n***** Server Information *****\n" );
        fprintf( destination , "** Server:  %s\n" , token[4] );
	fprintf( destination , "** Port:    %s\n" , token[2] );
        fprintf( destination , "** Game:    %s - %s\n" , token[6] , token[5] );
	fprintf( destination , "** Players: %s/%s\n" , token[7] , token[8] );
	fprintf( destination , "** Skill:   %s\n" , token[11] );
      }
      else
      {
        fprintf( destination , "\n***** Server Information *****\n" );
        fprintf( destination , "No information to display\n" );
      }
      break;
    }
    
    case QUAQUT_GAMEINFO:
    {
      if ( quaqut.buffer_reply_entriesnum[request] > 0 )
      {
        char* token[quaqut.buffer_reply_entriesnum[request]];
        int i;
	
	token[0] = (char*)quaqut_utils_strsep( quaqut.buffer_reply[request] , '\n' );
        for ( i = 1 ; i < quaqut.buffer_reply_entriesnum[request] ; i++ )
          token[i] = (char*)quaqut_utils_strsep( NULL , '\n' );
        
	fprintf( destination , "\n***** Game Information *****\n" );
	for ( i = 0 ; i < quaqut.buffer_reply_entriesnum[request] ; i = i + 2 )
	  fprintf( destination , "** %s: %s\n" , token[i] , token[i+1] );
      }
      else
      {
        fprintf( destination , "\n***** Game Information *****\n" );
        fprintf( destination , "No information to display\n" );
      }
      break;
    }
    
    case QUAQUT_PLAYERINFO:
    {
      if ( quaqut.buffer_reply_entriesnum[request] > 0 )
      {
        char* token[quaqut.buffer_reply_entriesnum[request]];
	int i;
	
	token[0] = (char*)quaqut_utils_strsep( quaqut.buffer_reply[request] , '\n' );
        for ( i = 1 ; i < quaqut.buffer_reply_entriesnum[request] ; i++ )
          token[i] = (char*)quaqut_utils_strsep( NULL , '\n' );
        
	fprintf( destination , "\n***** Players Information *****\n" );
	fprintf( destination , "** %-25s %-8s %-8s %s\n**\n" , "player name" , "score" , "ping" , "team" );

	for ( i = 0 ; i < quaqut.buffer_reply_entriesnum[request] ; i = i + 5 )
	  fprintf( destination , "** %-25s %-8s %-8s %c\n" , token[i+1] , token[i+3] , token[i+2] , token[i+4][0] );
      }
      else
      {
        fprintf( destination , "\n***** Player Information *****\n" );
        fprintf( destination , "No information to display\n" );
      }
      break;
    }
  
  }
}
