/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "quaqut.h"
#include "protocol.h"

#ifdef _WIN_32_
  void quaqut_network_initwsa( void );
  void quaqut_network_shutwsa( void );
#endif

void quaqut_network_initsocket( void )
{
  #ifdef _WIN_32_
    unsigned long val = 1;
    quaqut_network_initwsa();
  #endif
  
  quaqut.socket = socket( PF_INET , SOCK_DGRAM , IPPROTO_UDP );
  
  /* Non blocking socket, implementation differs from UNIX to Windows */
  #ifdef _WIN_32_
    ioctlsocket( quaqut.socket , FIONBIO , &val );
  #else
    fcntl( quaqut.socket , F_SETFL , fcntl( quaqut.socket , F_GETFL , 0 ) | O_NONBLOCK );
  #endif  
}

void quaqut_network_shutsocket( void )
{
  close( quaqut.socket );
  #ifdef _WIN_32_
    quaqut_network_shutwsa();
  #endif
}

int quaqut_network_lookup( void )
{
  /* Host name resolution */
  if ( !quaqut_network_initsockaddr( &(quaqut.server_name) , quaqut.server_host , quaqut.server_port ) )
  {
    quaqut_display_error( 0 , "Error, unknow host: %s\n" , quaqut.server_host );
    return 0;
  }
  else return 1;
}

void quaqut_network_process( int protocol , char command )
{
  if ( protocol == QUAQUT_PROTOCOL_UT2004 )
  {
    switch ( command )
    {
      case 'a':
      {
        quaqut_network_send( QUAQUT_PROTOCOL_UT2004 , QUAQUT_SERVERINFO );
        quaqut_network_send( QUAQUT_PROTOCOL_UT2004 , QUAQUT_GAMEPLAYERINFO );   
        break;
      }
    
      case 's':
        quaqut_network_send( QUAQUT_PROTOCOL_UT2004 , QUAQUT_SERVERINFO );
        break;
    
      case 'g':
        quaqut_network_send( QUAQUT_PROTOCOL_UT2004 , QUAQUT_GAMEINFO );
        break;
    
      case 'y':
        quaqut_network_send( QUAQUT_PROTOCOL_UT2004 , QUAQUT_PLAYERINFO );
        break;  
    }
    /* Now we get server responses, which depends on what we sent */
    quaqut_network_poll( QUAQUT_PROTOCOL_UT2004 );
  }
}


void quaqut_network_send( int protocol , int request )
{
  sendto( quaqut.socket , quaqut_query_string[protocol][request] , 
          sizeof(quaqut_query_string [protocol][request]) , 0 ,
	  (struct sockaddr *)&(quaqut.server_name) , sizeof(quaqut.server_name) );
}


void quaqut_network_poll( int protocol )
{
  /* We'll get responses here; their number is not always predictable when playerinfo
     is requested. When there aren't players, playerinfo may or may not be sent, since it
     may contain the score of teams in any case. What's worse, playerinfo may be carried in
     more than one UDP message, if there are many players and spectators (plus teams score). */
  
  /* Because of this we are working with select, which gathers all information available
     in a period of time (multiple playerinfo messages as well, if there are many). */
  fd_set set;
  struct timeval timeout;
  
  FD_ZERO( &set );
  FD_SET( quaqut.socket , &set );
  
  /* This will be user configurable, in tenth of second */
  timeout.tv_sec = quaqut.recv_timeout_sec;
  timeout.tv_usec = quaqut.recv_timeout_decsec * 100000;
  
  while ( select( FD_SETSIZE , &set , NULL , NULL , &timeout ) > 0 )
    quaqut_network_recv( protocol );
  
  FD_CLR( quaqut.socket , &set );
}


void quaqut_network_recv( int protocol )
{
  /* 2048 bytes should be more than enough */
  unsigned char buffer[2048];
  int fromlen = sizeof(quaqut.server_name);
  int buffer_size;
  memset( buffer , '\0' , 2048 );
  buffer_size = recvfrom( quaqut.socket , buffer , 2047 , 0 , (struct sockaddr *)&(quaqut.server_name) , &fromlen );

  /* Now we place the temporary buffer in the correct quaqut buffer */
  switch ( buffer[4] )
  {
    case 0x00:
      memcpy( quaqut.buffer_reply[QUAQUT_SERVERINFO] , buffer , 2047 );
      quaqut.buffer_reply_size[QUAQUT_SERVERINFO] = buffer_size;
      break;
    
    case 0x01:
      memcpy( quaqut.buffer_reply[QUAQUT_GAMEINFO] , buffer , 2047 );
      quaqut.buffer_reply_size[QUAQUT_GAMEINFO] = buffer_size;
      break;
    
    case 0x02:
      /* Buffer can be copied more than once, if there's more than one UDP message for playerinfo.
         So we must keep in count the quaqut.buffer_reply_size for playerinfo when using memcpy */
      if ( quaqut.buffer_reply_size[QUAQUT_PLAYERINFO] + buffer_size <= 2047 )
      {
        /* We copy buffers excluding the first five bytes which aren't anymore useful */
        memcpy( &(quaqut.buffer_reply[QUAQUT_PLAYERINFO][quaqut.buffer_reply_size[QUAQUT_PLAYERINFO]]) ,
	        &buffer[5] , 2047 - quaqut.buffer_reply_size[QUAQUT_PLAYERINFO] );
        quaqut.buffer_reply_size[QUAQUT_PLAYERINFO] += (buffer_size -  5);
      }
      break;
  }

}


int quaqut_network_initsockaddr( struct sockaddr_in* name , const char* hostname , unsigned short int port )
{
  struct hostent *hostinfo;
  unsigned int addr, flag_ipaddr = 0;
  
  if ( !quaqut_utils_isipaddress( hostname ) )
  {
    hostinfo = gethostbyname(hostname);
    if ( hostinfo == NULL )
    {
      /* Something was wrong with host resolution */
      return 0;
    }
  }
  else
  {
    addr = inet_addr( hostname );
    flag_ipaddr = 1;
  } 
   
  name->sin_family = AF_INET;
  name->sin_port = htons(port);
  if ( flag_ipaddr ) name->sin_addr = *(struct in_addr *) (char*)&addr;
  else name->sin_addr = *(struct in_addr *) hostinfo->h_addr_list[0];
  return 1;
}


#ifdef _WIN_32_
void quaqut_network_initwsa( void )
{
  WORD wVersionRequested = MAKEWORD( 2, 2 );
  WSADATA wsaData;
  WSAStartup( wVersionRequested , &wsaData );
}

void quaqut_network_shutwsa( void )
{
  WSACleanup();
}
#endif
