/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <stdarg.h>
#include "quaqut.h"

const long int exp16[7] = { 0x0000001 , 0x0000010 , 0x0000100 , 0x0001000 ,
                            0x0010000 , 0x0100000 , 0x1000000 };

struct {
  int index;
  int original_length;
  char* current_string;
} sep;


/* These are simple function for values initialization */

void quaqut_utils_initvalues( void )
{
  quaqut_utils_cleanbuffers();
  quaqut.recv_timeout_sec = QUAQUT_QUERY_STANDARD_TIMEOUT_SEC;
  quaqut.recv_timeout_decsec = QUAQUT_QUERY_STANDARD_TIMEOUT_DECSEC;
  quaqut.server_host = NULL;
  quaqut.server_port = 0;
  quaqut.options = 0;
}

void quaqut_utils_cleanbuffers( void )
{
  int i;
  for ( i = 0 ; i < 3 ; i++ )
  {
    quaqut.buffer_reply_size[i] = 0;
    quaqut.buffer_reply_entriesnum[i] = 0;
    memset( quaqut.buffer_reply[i] , '\0' , 2048 );
  }
}


/* This function is called for cleanup after SIGTERM and SIGINT */

void quaqut_utils_cleanup( int signum )
{
  /* Call socket cleanup for signal termination too */
  quaqut_network_shutsocket();
  /* Now issue the same signal with default behaviour */
  signal( signum , SIG_DFL );
  raise( signum );
}


/* This is a simple function that will convert sequence of
   bytes (0x??) in the real exadecimal number they represent */
   
int quaqut_utils_gethex( unsigned int count , ... )
{
  int hex = 0x00;
  int i;

  /* exp16[] overflows won't happen anyway */
  if ( count > 4 )
  {
    return 0x00;
  }
  else
  {
    va_list ap;
    va_start( ap , count );
    
    for ( i = 0 ; i < count ; i++ )
    {
      hex += va_arg( ap , int ) * exp16[(i*2)];
    }
    
    va_end( ap );
    return hex;
  }
}


/* This is a simple functions which removes newlines '\n', escapes (0x1b) and some
   other stuff (0xf5) from a string, and returns the length of the string after strip */
   
int quaqut_utils_stripn( unsigned char* buffer , int len )
{
  int i, k;
  for ( i = 0 ; i < len ; i++ )
  {
    if (( buffer[i] == '\n' ) || ( buffer[i] == 0x1b ) || ( buffer[i] == 0xf5 ))
    {
      for ( k = i ; k < (len - 1) ; k++ )
      {
        buffer[k] = buffer[k+1];
      }
      len--;
      i--;
    }
  }
  return len;
}


/* This is a simple function to minimally check if a string contains an IP address */

int quaqut_utils_isipaddress( const char* string )
{
  int i = 0 , dc = 0 , pc = 0;
  int max = strlen( string );
  
  for ( i = 0 ; i < max ; i++ )
  {
    if ( isdigit( string[i] ) )
    {
      dc++;
      if ( dc > 3 ) return 0;
    }
    else if (( string[i] == '.' ) && ( dc > 0 ) && ( pc < 3 ))
    {
      dc = 0; pc++;
    }
    else return 0;
  }
  return 1;
}


/* This function will read two bytes from a buffer, starting from the one
   pointed by index, and store the decimal value they represent in another buffer */
   
int quaqut_utils_bufferize2bytes( unsigned char* buffer_destination ,
                                  unsigned char* buffer_source , int index )
{
  int nbuffer;
  char tbuffer[8];
  
  nbuffer = quaqut_utils_gethex( 2 , buffer_source[index] , buffer_source[index + 1] );
  snprintf( tbuffer , 7 , "%d\n" , nbuffer );
  strncat( buffer_destination , tbuffer , 7 );
  
  return 1;
}


/* This function will read four bytes from a buffer, starting from the one
   pointed by index, and store the decimal value they represent in another buffer;
   if withcomplement is active, result can be negative and needs special processing */

int quaqut_utils_bufferize4bytes( unsigned char* buffer_destination ,
                                  unsigned char* buffer_source ,
                                  int index , int withcomplement )
{
  int nbuffer;
  char tbuffer[12];
  
  nbuffer = quaqut_utils_gethex( 4 , buffer_source[index] , buffer_source[index + 1] ,
                                 buffer_source[index + 2] , buffer_source[index + 3] );
  
  if (( withcomplement ) && ( buffer_source[index + 3] >= 0x80 ))
  {
    /* The most significative bit is set, this should be a negative number */
    nbuffer = 0xffffffff - nbuffer + 1;
    nbuffer *= -1;
  }
  
  snprintf( tbuffer , 11 , "%d\n" , nbuffer );
  strncat( buffer_destination , tbuffer , 11 );
  
  return 1;
}


/* This function will read a sequence of bytes from a buffer, the length
   of sequence is given by the value of the first byte, pointed by index;
   last, it stores the string represented by the sequence in another buffer */

int quaqut_utils_bufferizestring( unsigned char* buffer_destination ,
                                  unsigned char* buffer_source , int index )
{
  char tbuffer[256];
  int tbuffer_len;
  
  snprintf( tbuffer , buffer_source[index] , "%s" , &(buffer_source[index + 1]) );
  tbuffer_len = quaqut_utils_stripn( tbuffer , buffer_source[index] );
  strncat( buffer_destination , tbuffer , tbuffer_len );
  strncat( buffer_destination , "\n" , 1 );
  
  return 1;
}


/* This function has been written by myself to replace strsep(), which is not
   in POSIX standard. Strings should be NULL terminated to work with this code */

char* quaqut_utils_strsep( char* string , char delimiter )
{
  int i = 0;

  if ( string != NULL )
  {
    sep.index = 0;
    sep.current_string = string;
    sep.original_length = strlen( string );
  }
  
  if ( sep.index >= sep.original_length ) return NULL;
  
  for ( i = sep.index ; i <= sep.original_length ; i++ )
  {
    if ( sep.current_string[i] == delimiter )
    {
      int oldindex = sep.index;
      sep.index = i + 1;
      sep.current_string[i] = '\0';
      return &sep.current_string[oldindex];
    }
    else if ( sep.current_string[i] == '\0' )
    {
      /* End of the string, let's get the last token */
      /* We also make an extra check, which is probably not necessary but it won't hurt */
      if ( sep.current_string[i-1] != '\0' )
      {
        int oldindex = sep.index;
        sep.index = i + 1;
        sep.current_string[i] = '\0';
        return &sep.current_string[oldindex];
      }
      else return NULL;
    }
  }
  /* If string is correctly NULL terminated, it's not possible to get here */
  return NULL;
}
